//
// Scene class
//
#ifndef AMBROSESCENE_H
#define AMBROSESCENE_H

class AmbroseObject;
class AmbroseTexture;
class AmbroseMaterial;
class AmbroseMesh;
class AmbroseLight;
class AmbroseCamera;
class AmbroseAnimation;

class AmbroseScene
{
public:
    // Ctor
    AmbroseScene();
    // Dtor
    virtual ~AmbroseScene();

    // Load scene from file
    int load(const char *aFilename);

    // Apply animations to all objects, tweening between two frames
    void applyAnimations(int aFrame, int aNextFrame, float aTween);
    // Apply animations to all objects, animation position 0..1
    void applyAnimations(float pos);
    // Set up lights for rendering
    void setupLights();
    // Set up camera for rendering
    void setupCamera(int aCameraIndex, float aAspect);
    // Render the scene
    void render();

    // Number of meshes in the scene
    int mMeshes;
    // Number of lights in the scene
    int mLights;
    // Number of cameras in the scene
    int mCameras;
    // Number of animation controllers in the scene
    int mAnimations;
    // Number of frames in the animations 
    int mFrames;
    // Number of available materials
    int mMaterials;
    // Number of available textures
    int mTextures;

    // Array of texture objects
    AmbroseTexture *mTexture;
    // Array of material objects
    AmbroseMaterial *mMaterial;
    // Array of mesh objects
    AmbroseMesh *mMesh;
    // Array of light objects
    AmbroseLight *mLight;
    // Array of camera objects
    AmbroseCamera *mCamera;
    // Array of animation objets
    AmbroseAnimation *mAnimation;

    // Find object (mesh, camera, light) by name
    AmbroseObject *find(const char *aName); 
    // Find object (mesh, camera, light) by id
    AmbroseObject *find(int aId);
    // Find mesh by name
    AmbroseMesh *findMesh(const char *aName);
    // Find mesh by id
    AmbroseMesh *findMesh(int aId);
    // Find light by name
    AmbroseLight *findLight(const char *aName);
    // Find light by id
    AmbroseLight *findLight(int aId);
    // Find camera by name
    AmbroseCamera *findCamera(const char *aName);
    // Find camera by id
    AmbroseCamera *findCamera(int aId);
    // Find material by name
    AmbroseMaterial *findMaterial(const char *aName);
    // Find material by id
    AmbroseMaterial *findMaterial(int aId);
    // Find texture by name
    AmbroseTexture *findTexture(const char *aName);
    // Find texture by id
    AmbroseTexture *findTexture(int aId);
};

#endif