//
// Texture slot class
//
#ifndef AMBROSETEXTURESLOT_H
#define AMBROSETEXTURESLOT_H

/* Blender texture blend modes:
* MIX - mix texture with value
* MULTIPLY - multiply texture with value
* ADD - add texture to value
* SUBTRACT - subtract texture from value
* DIVIDE - divide value by texture
* DARKEN - replace value with texture if texture is darker
* DIFFERENCE - difference of texture from value
* LIGHTEN - replace value with texture if texture is lighter
* SCREEN - 'screen' mode 
*/

/* Blender texture 'map to' modes:
* COL - Make the texture affect the basic colour of the material
* NOR - Make the texture affect the rendered normal
* CSP - Make the texture affect the specularity colour
* CMIR - Make the texture affect the mirror colour
* REF - Make the texture affect the diffuse reflectivity value
* SPEC - Make the texture affect the specularity value
* HARD - Make the texture affect the hardness value
* ALPHA - Make the texture affect the alpha value
* EMIT - Make the texture affect the emit value
* RAYMIR - Make the texture affect the mirror reflectivity value
*  DISP - Make the texture displace the mesh
* TRANSLU - Make the texture affect the translucency value
* AMB - Make the texture affect the ambient value
* WARP - Make the texture affect texture coordinates for the following textures 
*/

class AmbroseTextureSlot
{
public: 
    // Ctor
    AmbroseTextureSlot();
    // Dtor
    virtual ~AmbroseTextureSlot();
    // Blend mode for this texture
    int mBlendMode;
    // Use mode for this texture
    int mUseMode;
    // Pointer to the texture
    AmbroseTexture *mTex;
    // Load texture slot from file
    int load(AmbroseScene *aScene, FILE *f);
};

#endif