#include "Ambrose.h"

AmbroseScene::AmbroseScene()
{
    mMeshes = 0;
    mLights = 0;
    mCameras = 0;
    mAnimations = 0;
    mFrames = 0;
    mMaterials = 0;
    mTextures = 0;

    mTexture = NULL;
    mMaterial = NULL;
    mMesh = NULL;
    mLight = NULL;
    mCamera = NULL;
    mAnimation = NULL;  
}

AmbroseScene::~AmbroseScene()
{
    delete[] mTexture;
    delete[] mMaterial;
    delete[] mMesh;
    delete[] mLight;
    delete[] mCamera;
    delete[] mAnimation;  

    mTexture = NULL;
    mMaterial = NULL;
    mMesh = NULL;
    mLight = NULL;
    mCamera = NULL;
    mAnimation = NULL;  
}

AmbroseMesh * AmbroseScene::findMesh(int aId)
{
    int i;
    for (i = 0; i < mMeshes; i++)
        if (mMesh[i].mId == aId)
            return &mMesh[i];
    return NULL;
}

AmbroseLight * AmbroseScene::findLight(int aId)
{
    int i;
    for (i = 0; i < mLights; i++)
        if (mLight[i].mId == aId)
            return &mLight[i];
    return NULL;
}

AmbroseCamera * AmbroseScene::findCamera(int aId)
{
    int i;
    for (i = 0; i < mCameras; i++)
        if (mCamera[i].mId == aId)
            return &mCamera[i];
    return NULL;
}

AmbroseObject * AmbroseScene::find(int aId)
{
    AmbroseObject *o;
    o = findMesh(aId);
    if (o) return o;
    o = findLight(aId);
    if (o) return o;
    return findCamera(aId);
}


AmbroseMesh * AmbroseScene::findMesh(const char *aName)
{
    int i;
    for (i = 0; i < mMeshes; i++)
        if (!strcmp(mMesh[i].mName, aName))
            return &mMesh[i];
    return NULL;
}

AmbroseLight * AmbroseScene::findLight(const char *aName)
{
    int i;
    for (i = 0; i < mLights; i++)
        if (!strcmp(mLight[i].mName, aName))
            return &mLight[i];
    return NULL;
}

AmbroseCamera * AmbroseScene::findCamera(const char *aName)
{
    int i;
    for (i = 0; i < mCameras; i++)
        if (!strcmp(mCamera[i].mName, aName))
            return &mCamera[i];
    return NULL;
}

AmbroseObject * AmbroseScene::find(const char *aName)
{
    AmbroseObject *o;
    o = findMesh(aName);
    if (o) return o;
    o = findLight(aName);
    if (o) return o;
    return findCamera(aName);
}


AmbroseMaterial * AmbroseScene::findMaterial(const char *aName)
{
    int i;
    for (i = 0; i < mMaterials; i++)
        if (!strcmp(mMaterial[i].mName, aName))
            return &mMaterial[i];
    return NULL;
}

AmbroseTexture * AmbroseScene::findTexture(const char *aName)
{
    int i;
    for (i = 0; i < mTextures; i++)
        if (!strcmp(mTexture[i].mName, aName))
            return &mTexture[i];
    return NULL;
}

AmbroseMaterial * AmbroseScene::findMaterial(int aId)
{
    int i;
    for (i = 0; i < mMaterials; i++)
        if (mMaterial[i].mId == aId)
            return &mMaterial[i];
    return NULL;
}

AmbroseTexture * AmbroseScene::findTexture(int aId)
{
    int i;
    for (i = 0; i < mTextures; i++)
        if (mTexture[i].mId == aId)
            return &mTexture[i];
    return NULL;
}

int AmbroseScene::load(const char *aFilename)
{
    int i;
    FILE * f = fopen(aFilename, "rb");

    // Check the magic header bytes
    if (readInt(f) != '\01bmA')
    {
        return 0;
    }

    mMeshes = readInt(f);
    mLights = readInt(f);
    mCameras = readInt(f);
    mAnimations = readInt(f);
    mMaterials = readInt(f);
    mTextures = readInt(f);
    mFrames = readInt(f);

    mTexture = new AmbroseTexture[mTextures];
    if (mTexture == NULL)
        return 0;
    for (i = 0; i < mTextures; i++)
    {
        if (!mTexture[i].load(this, f))
            return 0;
        // Textures and materials have linear id:s, not stored in file
        mTexture[i].mId = i + 1;
    }

    mMaterial = new AmbroseMaterial[mMaterials];
    if (mMaterial == NULL)
        return 0;
    for (i = 0; i < mMaterials; i++)
    {
        if (!mMaterial[i].load(this, f))
            return 0;
        // Textures and materials have linear id:s, not stored in file
        mMaterial[i].mId = i + 1;
    }

    mMesh = new AmbroseMesh[mMeshes];
    if (mMesh == NULL)
        return 0;
    for (i = 0; i < mMeshes; i++)
    {
        if (!mMesh[i].load(this, f))
            return 0;

        // Activate VBOs etc
        mMesh[i].mVB.optimize();
    }

    mLight = new AmbroseLight[mLights];
    if (mLight == NULL)
        return 0;
    for (i = 0; i < mLights; i++)
    {
        if (!mLight[i].load(this, f))
            return 0;
    }

    mCamera = new AmbroseCamera[mCameras];
    if (mCamera == NULL)
        return 0;
    for (i = 0; i < mCameras; i++)
    {
        if (!mCamera[i].load(this, f))
            return 0;
    }

    mAnimation = new AmbroseAnimation[mAnimations];
    if (mAnimation == NULL)
        return 0;
    for (i = 0; i < mAnimations; i++)
    {
        if (!mAnimation[i].load(this, f))
            return 0;   
    }

    return 1;
}

void AmbroseScene::render()
{
    int i;
    for (i = 0; i < mMeshes; i++)
    {
        if (mMesh[i].mMaterial)
        {
            AmbroseTextureSlot *t = mMesh[i].mMaterial->mSlot[0];
            if (t == NULL)
                glDisable(GL_TEXTURE_2D);
            else
            {
                glEnable(GL_TEXTURE_2D);
                glBindTexture(GL_TEXTURE_2D, t->mTex->mTextureHandle);
            }
            glColor3f(mMesh[i].mMaterial->mDiffuse.r, 
                mMesh[i].mMaterial->mDiffuse.g, 
                mMesh[i].mMaterial->mDiffuse.b);
        }
        else
        {
            // no material
            glDisable(GL_TEXTURE_2D);
            glColor3f(1,1,1);
        }
        mMesh[i].mVB.activate();

        glMatrixMode( GL_MODELVIEW );
        glPushMatrix();

        glMultMatrixf(mMesh[i].mMatrix.v);

        glFrontFace(GL_CCW);

        mMesh[i].mVB.render();

        glPopMatrix();
    }
}

void AmbroseScene::applyAnimations(int aFrame, int aNextFrame, float aTween)
{
    int i;
    for (i = 0; i < mAnimations; i++)
    {
        matrix4x4 m;
        float jframe = 1.0f - aTween;
        int j;
        for (j = 0; j < 4*4; j++)
            m.v[j] = mAnimation[i].mFrame[aFrame].v[j] * jframe + 
            mAnimation[i].mFrame[aNextFrame].v[j] * aTween;
        *mAnimation[i].mTargetPtr = m;
    }
}

void AmbroseScene::applyAnimations(float pos)
{
    pos = (float)fmod((double)pos, 1.0) * mFrames;
    int frame = (int)floor(pos);
    int nextframe = (frame + 1) % mFrames;
    float iframe = pos - frame;
    applyAnimations(frame, nextframe, iframe);
}

void AmbroseScene::setupLights()
{
    int i;
    if (mLights)
    {       
        for (i = 0; i < mLights; i++)
        {
            glEnable(GL_LIGHT0 + i);
            GLfloat lightdata[4];

            // Set light position (read from light matrix)
            lightdata[0] = mLight[i].mMatrix.v[3*4+1];
            lightdata[1] = mLight[i].mMatrix.v[3*4+2];
            lightdata[2] = mLight[i].mMatrix.v[3*4+3];
            lightdata[3] = 1;
            glLightfv(GL_LIGHT0 + i, GL_POSITION, lightdata);

            // Set light color
            lightdata[0] = mLight[i].mColor.r;
            lightdata[1] = mLight[i].mColor.g;
            lightdata[2] = mLight[i].mColor.b;
            lightdata[3] = 1;
            glLightfv(GL_LIGHT0 + i, GL_AMBIENT, lightdata);
        }
        glEnable(GL_LIGHTING);
    }
    else
    {
        glDisable(GL_LIGHTING);
    }
}

void AmbroseScene::setupCamera(int aCameraIndex, float aAspect)
{
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(mCamera[aCameraIndex].mLens, aAspect, mCamera[aCameraIndex].mNearClip, mCamera[aCameraIndex].mFarClip);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    glMultMatrixf(mCamera[aCameraIndex].mMatrix.v); 
}
