#include <SDL.h>
#include <SDL_image.h>
#include <math.h>
#include "glee.h"


GLuint loadTexture(char * aFilename)
{
    // Create OpenGL texture handle and bind it to use
    GLuint texname;
    glGenTextures(1,&texname);
    glBindTexture(GL_TEXTURE_2D,texname);

    // Load texture using SDL_Image
    SDL_Surface *temp = IMG_Load(aFilename);

    // Set up opengl-compatible pixel format
    SDL_PixelFormat pf;
    pf.palette = NULL;
    pf.BitsPerPixel = 32;
    pf.BytesPerPixel = 4;
    pf.alpha = 0;
    pf.colorkey = 0;
    pf.Rmask = 0x000000ff;
    pf.Rshift = 0;
    pf.Rloss = 0;
    pf.Gmask = 0x0000ff00;
    pf.Gshift = 8;
    pf.Gloss = 0;
    pf.Bmask = 0x00ff0000;
    pf.Bshift = 16;
    pf.Bloss = 0;
    pf.Amask = 0xff000000;
    pf.Ashift = 24;
    pf.Aloss = 0;

    // Convert texture to said format
    SDL_Surface *tm = SDL_ConvertSurface(temp, &pf,SDL_SWSURFACE);

    // Cleanup
    SDL_FreeSurface(temp);

    // Lock the converted surface for reading
    SDL_LockSurface(tm);

    // Tell OpenGL to read the texture
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, tm->w, tm->h, 0, GL_RGBA, GL_UNSIGNED_BYTE, (GLvoid*)tm->pixels);

    // Unlock..
    SDL_UnlockSurface(tm);

    // and cleanup.
    SDL_FreeSurface(tm);

    // Set up texture parameters
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR); // Linear Filtering
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // Linear Filtering

    // Note: no mipmaps were generated!

    return texname;
}
