#include <SDL.h>
#include "glee.h"
#include "VertexBuffer.h"

void VertexBuffer::optimize()
{
    if (GLEE_ARB_vertex_buffer_object)
    {
        if (mVertex)
        {
            glGenBuffers(1, &mVertexVBO);   
            glBindBuffer(GL_ARRAY_BUFFER, mVertexVBO);
            glBufferData(GL_ARRAY_BUFFER, mVertices * sizeof(float) * 3, mVertex, GL_STATIC_DRAW);
        }
        if (mNormal)
        {
            glGenBuffers(1, &mNormalVBO);
            glBindBuffer(GL_ARRAY_BUFFER, mNormalVBO);
            glBufferData(GL_ARRAY_BUFFER, mVertices * sizeof(float) * 3, mNormal, GL_STATIC_DRAW);
        }

        int i;
        for (i = 0; i < 4; i++)
        {
            if (mTexcoord[i])
            {
                glGenBuffers(1, &mTexCoordVBO[i]);
                glBindBuffer(GL_ARRAY_BUFFER, mTexCoordVBO[i]);
                glBufferData(GL_ARRAY_BUFFER, mVertices * sizeof(float) * 2, mTexcoord[i], GL_STATIC_DRAW);
            }
        }

        if (mIndex)
        {
            glGenBuffers(1, &mIndexVBO);
            glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, mIndexVBO);
            glBufferData(GL_ELEMENT_ARRAY_BUFFER, mIndices * sizeof(GLuint), mIndex, GL_STATIC_DRAW);     
        }

    }
}

void VertexBuffer::render_normals(float aScale)
{
    GLfloat *temp = new GLfloat[mVertices * 3 * 2];
    int i;
    for (i = 0; i < mVertices; i++)
    {
        temp[i * 6 + 0] = mVertex[i * 3 + 0];
        temp[i * 6 + 1] = mVertex[i * 3 + 1];
        temp[i * 6 + 2] = mVertex[i * 3 + 2];

        temp[i * 6 + 3] = mVertex[i * 3 + 0] + mNormal[i * 3 + 0] * aScale;
        temp[i * 6 + 4] = mVertex[i * 3 + 1] + mNormal[i * 3 + 1] * aScale;
        temp[i * 6 + 5] = mVertex[i * 3 + 2] + mNormal[i * 3 + 2] * aScale;
    }


    glBindBuffer(GL_ARRAY_BUFFER, 0);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDisableClientState(GL_NORMAL_ARRAY);
    glEnableClientState(GL_VERTEX_ARRAY);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_LIGHTING);
    glColor4f(1,0,0,1);
    glVertexPointer(3, GL_FLOAT, 0, temp);            
    glDrawArrays(GL_LINES,0,mVertices * 2);
    delete[] temp;
}


void VertexBuffer::activate()
{
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);
    glEnableClientState(GL_NORMAL_ARRAY);

    if (mVertexVBO)
    {
        glBindBuffer(GL_ARRAY_BUFFER, mVertexVBO);
        glVertexPointer(3, GL_FLOAT, 0, 0);
    }
    else
    {
        glBindBuffer(GL_ARRAY_BUFFER, 0);
        glVertexPointer(3, GL_FLOAT, 0, mVertex);
    }
    if (mNormalVBO)
    {
        glBindBuffer(GL_ARRAY_BUFFER, mNormalVBO);
        glNormalPointer(GL_FLOAT, 0, 0);
    }
    else
    {
        glBindBuffer(GL_ARRAY_BUFFER, 0);
        glNormalPointer(GL_FLOAT, 0, mNormal);
    }

    int i;
    for (i = 0; i < 4; i++)
    {
        if (mTexcoord[i] || mTexCoordVBO[i])
        {
            glClientActiveTexture(GL_TEXTURE0_ARB + i);
            if (mTexCoordVBO[i])
            {
                glBindBuffer(GL_ARRAY_BUFFER, mTexCoordVBO[i]);
                glTexCoordPointer(2, GL_FLOAT, 0, 0);
            }
            else
            {
                glBindBuffer(GL_ARRAY_BUFFER, 0);
                glTexCoordPointer(2, GL_FLOAT, 0, mTexcoord[i]);
            }
        }
    }
    if (mIndexVBO)
    {
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, mIndexVBO);
    }
    else
    {
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, 0);
    }
    glClientActiveTexture(GL_TEXTURE0_ARB);
}

void VertexBuffer::render()
{
    if (mIndexVBO)
        glDrawElements(mPrimitiveType,mIndices,GL_UNSIGNED_INT,0);
    else
        glDrawElements(mPrimitiveType,mIndices,GL_UNSIGNED_INT,mIndex);
}

VertexBuffer::VertexBuffer()
{
    mVertex = NULL;
    mNormal = NULL;
    mIndex = NULL;
    mIndices = 0;
    mPrimitiveType = 0;
    mTexcoord[0] = mTexcoord[1] = mTexcoord[2] = mTexcoord[3] = 0;
    mVertexVBO = 0;
    mNormalVBO = 0;
    mTexCoordVBO[0] = mTexCoordVBO[1] = mTexCoordVBO[2] = mTexCoordVBO[3] = 0;
    mIndexVBO = 0;
}

VertexBuffer::~VertexBuffer()
{
    int i;

    if (mVertexVBO)
        glDeleteBuffers(1, &mVertexVBO);
    if (mNormalVBO)
        glDeleteBuffers(1, &mNormalVBO);

    for (i = 0; i < 4; i++)
    {
        if (mTexCoordVBO[i])
            glDeleteBuffers(1, &mTexCoordVBO[i]);
    }
    if (mIndexVBO)
        glDeleteBuffers(1, &mIndexVBO);
    delete[] mVertex;
    mVertex = NULL;

    delete[] mNormal;
    mNormal = NULL;

    delete[] mIndex;
    mIndex = NULL;

    mIndices = 0;
    mPrimitiveType = 0;

    for (i = 0; i < 4; i++)
    {
        delete[] mTexcoord[i];
        mTexcoord[i] = NULL;
    }
}
